
# Functor class provides a way to do psuedo-namspaces
# This is useful for a number of methods.
# This is basically the class in Calibre but is included
# here to remove dependency.

class Functor

  # remove all non-essential methods
  EXCLUDE = /^(__|instance|null$|inspect$|dup$)/
  class << self
    def hide(name)
      if method_defined?(name) and name !~ EXCLUDE
        undef_method( name )
      end
    end
  end
  instance_methods.each { |m| hide(m) }

  # primary functionality
  def initialize(*obj, &func)
    @obj = obj
    @func = func
  end

  def method_missing(op, *args)
    if @obj.empty?
      @func.call(op, *args)
    else
      @func.call(op, *(@obj + args))
    end
  end

  # prevent new methods from being added
  class << self
    def method_added( name )
      hide( name )
    end
  end

end
